package org.heliostat;

public class SunTracking {

	public double latitude, longitude, targetAzimuth, targetElevation;
	public int timeZone, month, day, hour, minutes, seconds;
	
	public static double PI = 3.1415926535897932384626433832795028841971;
	public static double DR = 180 / PI; //degree / radian factor;
	public static double W = 2 * PI / 365; // earth's mean orbital angular speed in radians/day 
	public static double WR = PI / 12; // earth's speed of rotation relative to sun (radians/hour) 
	public static double C = -23.45 / DR; // reverse angle of earth's axial tilt in radians 
	public static double ST = Math.sin(C); // sine of reverse tilt 
	public static double CT = Math.cos(C); // cosine of reverse tilt 
	public static double E2 = 2 * 0.0167; // twice earth's orbital eccentricity 
	public static double SN = 10 * W; // 10 days from December solstice to New Year (Jan 1) 
	public static double SP = 12 * W; // 12 days from December solstice to perihelion 

	double rad2deg(double radians)
	{
	    return (180.0/PI)*radians;
	}

	double deg2rad(double degrees)
	{
	    return (PI/180.0)*degrees;
	}

	double ang(double x, double y)
	{
		double angle=0;
		
		if (x > 0)
		  angle = Math.atan(y/x);
		else if (x < 0)
		  angle = Math.atan(y/x) + PI;
		else if (y > 0)
		  angle = 1 * PI / 2;
		else if (y < 0)
		  angle = -1 * PI / 2;
		
		return angle;
	}

	
	public void P2C(double AZ, double EL, double[] coords)
	{
		double c;
		
		coords[2]=Math.sin(EL);
		c = Math.cos(EL);
		coords[0] = c * Math.sin(AZ);
		coords[1] = c * Math.cos(AZ);
	}

	void C2P(double x, double y, double z, double[] azimuthElevation)
	{
		azimuthElevation[1] = ang(Math.sqrt(x * x + y * y), z);
		azimuthElevation[0] = ang(y, x);
		if (azimuthElevation[0] < 0)
			azimuthElevation[0] = azimuthElevation[0] + 2 * PI;
	}
	
	public void calculate (double[] solarLocation, double[] mirrorLocation)
	{
		double A, B, SL, D, CL, DC, LD, LT, LG; 
		double[] solarAzimuthElevation=new double[2];
		double[] mirrorAzimuthElevation=new double [2];
		double[] targetAzimuthElevation=new double [2];
		double[] targetCoords=new double [3];
		double[] solarCoords=new double [3];
		
		//User input. Note: For brevity, no error checks on user inputs 
		LT = deg2rad(latitude);
		LG = deg2rad(longitude);
		targetAzimuthElevation[0] = deg2rad(targetAzimuth);
		targetAzimuthElevation[1] = deg2rad(targetElevation);

		CL = PI / 2 - LT; // co-latitude 
		D = (int)(30.6 * ((month + 9) % 12) + 58.5 + day) % 365; // day of year (D = 0 on Jan 1)
		A = W * D + SN; // orbit angle since solstice at mean speed 
		B = A + E2 * Math.sin(A - SP); // angle with correction for eccentricity 
		C = (A - Math.atan(Math.tan(B) / CT)) / PI;
		SL = PI * (C - Math.floor(C + .5)); // solar longitude relative to mean position 
		C = ST * Math.cos(B);
		DC = Math.atan(C / Math.sqrt(1 - C * C)); // solar declination (latitude) 
		LD = (hour - timeZone + minutes / 60.0) * WR + SL + LG; // longitude difference 

		P2C(LD, DC, solarCoords); // polar axis (perpend'r to azimuth plane)
		C2P(solarCoords[1],solarCoords[2],solarCoords[0], solarAzimuthElevation); // horizontal axis 
		P2C(solarAzimuthElevation[0] + CL, solarAzimuthElevation[1], solarCoords); // rotate by co-latitude 
		double tmp_x, tmp_y, tmp_z;
		tmp_x=solarCoords[0];
		tmp_y=solarCoords[1];
		tmp_z=solarCoords[2];
		solarCoords[0]=tmp_z;
		solarCoords[1]=tmp_x;
		solarCoords[2]=tmp_y;
		
		// Sun's position
		C2P(solarCoords[0], solarCoords[1], solarCoords[2], solarAzimuthElevation); // vertical axis 
	  
		solarLocation[0]=rad2deg(solarAzimuthElevation[0]);
		solarLocation[1]=rad2deg(solarAzimuthElevation[1]);
		
		// Mirror aim direction
		P2C(targetAzimuthElevation[0], targetAzimuthElevation[1], targetCoords); 
		C2P(solarCoords[0]+targetCoords[0],solarCoords[1]+targetCoords[1],solarCoords[2]+targetCoords[2], mirrorAzimuthElevation);
		mirrorLocation[0]=rad2deg(mirrorAzimuthElevation[0]);
		mirrorLocation[1]=rad2deg(mirrorAzimuthElevation[1]);
	}
	
	
}
